using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
#if NETCFDESIGNTIME
using System.Globalization;
using System.ComponentModel.Design.Serialization;
using System.Reflection;
#endif
namespace Gauges
{
	public enum LampPosition : int
	{
		UpperLeft	= 0,
		UpperRight	= 1,
		LowerLeft	= 2,
		LowerRight	= 3
	}

	public class Lamps : CollectionBase
	{	
		public void Add(Lamp newLamp)
		{
			List.Add(newLamp);
		}

		public Lamp this[int index]
		{
			get
			{
				return (Lamp)List[index];
			}
			set
			{
				List[index] = value;
			}
		}

	}

	#if NETCFDESIGNTIME
		[DesignTimeVisibleAttribute(false), 
		TypeConverter("LampConverter")]
	#endif
	public class Lamp : System.ComponentModel.Component
	{
		private Color offColor;
		private Color onColor;
		private Color bezelColor;
		private Point upperRight;
		private int bezelWidth;
		private int diameter;
		private bool state;
		private LampPosition position;
		private bool visible = false;

		public Lamp()
		{
			offColor = Color.DarkRed;
			onColor = Color.Red;
			bezelColor = Color.SlateGray;
			bezelWidth = 2;
			diameter = 0;
			upperRight = new Point(0,0);
			position = LampPosition.UpperLeft;
		}

		internal Lamp(Point UpperRight, int Diameter, LampPosition Position) : base()
		{
			upperRight = UpperRight;
			diameter = Diameter;
			position = Position;
		}
		
		internal Point UpperRight
		{
			get
			{
				return upperRight;
			}
			set
			{
				upperRight = value;
			}
		}

		internal int Diameter
		{
			get
			{
				return diameter;
			}
			set
			{
				diameter = value;
			}
		}

		public bool Visible
		{
			get
			{
				return visible;
			}
			set
			{
				visible = value;
			}
		}

		public LampPosition Position
		{
			get
			{
				return position;
			}
			set
			{
				position = value;
			}
		}

		public bool State
		{
			get
			{
				return state;
			}
			set
			{
				state = value;
			}
		}

		public Color OnColor
		{
			get
			{
				return onColor;
			}
			set
			{
				onColor = value;
			}
		}

		public Color OffColor
		{
			get
			{
				return offColor;
			}
			set
			{
				offColor = value;
			}
		}

		public Color BezelColor
		{
			get
			{
				return bezelColor;
			}
			set
			{
				bezelColor = value;
			}
		}

		public int BezelWidth
		{
			get
			{
				return bezelWidth;
			}
			set
			{
				bezelWidth = value;
			}
		}

		internal void Draw(Graphics TargetGraphics)
		{
			Brush brush;

			brush = new SolidBrush(bezelColor);
			TargetGraphics.FillEllipse(brush, upperRight.X - diameter, upperRight.Y - diameter, diameter, diameter);

			brush = (state) ? new SolidBrush(onColor) : new SolidBrush(offColor);
			TargetGraphics.FillEllipse(brush, upperRight.X - diameter + (bezelWidth / 2), upperRight.Y - diameter + (bezelWidth / 2), diameter - bezelWidth, diameter - bezelWidth);

			brush.Dispose();
		}

	}
#if NETCFDESIGNTIME
	internal class LampConverter : TypeConverter
	{
		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(InstanceDescriptor))
			{
				return true;
			}

			return base.CanConvertTo(context, destinationType);
		}

		public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
		{
			if (destinationType == typeof(InstanceDescriptor) && value is Lamp)
			{
				Lamp lamp = (Lamp)value;
              
				ConstructorInfo ci = typeof(Lamp).GetConstructor(new Type[] {});
				if (ci != null)
				{
					return new InstanceDescriptor(ci, null, false);
				}
			}
			return base.ConvertTo(context, culture, value, destinationType);
		}
	}
#endif
}
